define([
  'text!templates/LoyaltyCardAccountProfileView.html',
  'sharedhelpers/configurationProvider',
  'helpers/uiHelper',
  'libs/jquery.validate.min',
  'sharedhelpers/storageProvider',
  'sharedhelpers/cultureHelper',
  'helpers/validationHelper',
  'sharedhelpers/encryptionHelper'
], function (template, configurationProvider) {
    var view = Backbone.View.extend({
        template: _.template(template),
        localize: window.ObjectCollections.Localization.result,

        initialize: function () {
            var self = this;
            this.cultureHelper = new CultureHelper();
            this.zipValidation = this.cultureHelper.zipValidation();
            var phoneValidation = this.cultureHelper.phoneValidation();
            var dateValidation = this.cultureHelper.dateValidation();

            $.validator.addMethod('zipcode', function (value) {
                return value == '' || (self.zipValidation.regex.test(value.toUpperCase()));
            }, self.zipValidation.msg);

            $.validator.addMethod('phone', function (value) {
                return phoneValidation.regex.test(value);
            }, phoneValidation.msg);

            $.validator.addMethod('currentPass', function (value) {
                return value == '' || value.length > 3;
            }, this.localize.errorPasswordLength);

            var minPasswordLenght = window.appConfig.PaswordCriteria ? (window.appConfig.PaswordCriteria.minLenght || 4) : 4;
            var passwordLengthError = $.sprintf(this.localize.errorPasswordLength, minPasswordLenght);

            $.validator.addMethod('password-length', function (value) {
                return value.length >= minPasswordLenght;
            }, passwordLengthError);

            if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.uppercase) {
                $.validator.addMethod('password-uppercase', function (value) {
                    return value == '' || /^(?=.*[A-Z])/.test(value);
                }, this.localize.errorPasswordUppercase);
            }

            if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.lowercase) {
                $.validator.addMethod('password-lowercase', function (value) {
                    return value == '' || /^(?=.*[a-z])/.test(value);
                }, this.localize.errorPasswordLowercase);
            }

            if (window.appConfig.PaswordCriteria && window.appConfig.PaswordCriteria.number) {
                $.validator.addMethod('password-number', function (value) {
                    return value == '' || /^(?=.*\d)/.test(value);
                }, this.localize.errorPasswordNumber);
            }

            $.validator.addMethod('repeatPassword', function (value) {
                return value == $('#frmChangePassword #pass').val();
            }, this.localize.errorPasswordMismatch);

            $.validator.addMethod('date', function (value) {
                return dateValidation.regex.test(value.replace(/\s/g, '/'));
            }, dateValidation.msg);

            $.validator.addMethod('birthday', function (value) {
                var dateOfBirth = value.replace(/\s/g, '/');
                return DtHelper.dateDiffInYears(dateOfBirth) > window.appConfig.LoyaltySignUpRequiredAge;
            }, $.sprintf(this.localize.errorLoyaltySignUpRequiredAge, window.appConfig.LoyaltySignUpRequiredAge));

            $.validator.addClassRules({ phone: { phone: true } });

        },

        events: {
            'tap #saveLoyaltyCard': 'saveLoyaltyCard',
            'tap #deleteLoyaltyCard': 'deleteLoyaltyCard',
            'tap #changePassword': 'changePassword',
            'tap li[for="tab-value"]': 'switchTabs',
            'change #zipcode': 'zipcode',
            'change #favoriteTheater': 'onInputChange',
            'change #stateProvince': 'onInputChange',
            'change #gender': 'onInputChange',
            'change .basicInfo': 'onInputChange',
            'change #smsOptIn': 'handleSmsOptIn'
        },

        render: function () {
            var self = this;
            self.$el.html(self.template({
                localize: self.localize,
                model: self.model,
                zipValidation: self.zipValidation,
                theaters: window.ObjectCollections.AbridgedTheaterCollection.models || window.ObjectCollections.TheaterCollection.models,
                showChangePasswordInput: configurationProvider.usePwdForLoyaltyLogin()
            }));

            if (self.model.get('favoriteTheater')) {
                $(self.el).find('#favoriteTheater').val(self.model.get('favoriteTheater').value).attr('selected', true).siblings('option').removeAttr('selected');
            }

            if (self.model.get('stateProvince')) {
                $(self.el).find('#stateProvince').val(self.model.get('stateProvince').value).attr('selected', true).siblings('option').removeAttr('selected');
            }

            if (window.appConfig.EnableGenderForLoyalty && self.model.get('gender')) {
                $(self.el).find('#gender').val(self.model.get('gender').value).attr('selected', true).siblings('option').removeAttr('selected');
            }

            if (self.model.get('smsOptIn') && (self.model.get('smsOptIn').value == 1 || self.model.get('smsOptIn').value == 2)) {
                $(self.el).find('#smsOptIn').attr('checked', true);
            }

            //Header fix for iOS when soft keyboard is active
            UIHelper.iOSHeaderFix(self);

            return self;
        },

        onInputChange: function (event) {
            $("#" + event.target.id).valid();

            var item = this.model.get(event.target.name);

            if (item && item.isEditable) {
                $(event.target).addClass('field-updated');
            }
        },

        handleSmsOptIn: function (event) {
            var item = this.model.get(event.target.name);

            if (item && item.isEditable) {
                $(event.target).addClass('field-updated');
            }
        },

        saveLoyaltyCard: function (event) {
            event.preventDefault();

            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            var self = this;

            if ($("#frmLoyaltyCard").valid()) {
                $('[data-role=content]').css({ 'pointer-events': 'none' });
                $('#deleteLoyaltyCard').css({ 'pointer-events': 'none' });

                //in case there is a change in an input field currently in focus (change event is being fired after the tap event)
                //add the class to that field as well, or it will be skipped when saving
                var focused = document.activeElement;
                if (focused.hasAttribute('name') && focused.hasAttribute('value') && self.model.get(focused.name).value !== focused.value) {
                    $('#' + focused.id).addClass('field-updated');
                }

                //pick up all fields that have their values changed
                var inputSelector = $("#frmLoyaltyCard .field-updated");

                //Android 4.1.1 Fix
                if (inputSelector.length == 0)
                    inputSelector = $("#frmLoyaltyCard input");

                _.each(inputSelector, function (i) {
                    var item = self.model.get(i.name);

                    if (item && item.isEditable) {
                        if (i.name == 'phoneNumber') {
                            item.value = i.value.replace(/\D/g, '');
                        } else if (i.name == 'firstName' || i.name == 'lastName') {
                            item.value = $.trim(i.value);
                        } else if (i.name == 'dateOfBirth') {
                            item.value = i.value.replace(/\s/g, '/');
                        } else if ($(i).is('select')) {
                            item.value = $(i).find('option:selected').val();
                        } else if (i.name == 'smsOptIn') {
                            item.value = $(i).is(':checked') ? '2' : '0';
                        }
                        else {
                            item.value = i.value;
                        }

                        item.isUpdated = true;
                        self.model.set(item.name, item);
                    }
                });

                require(['shared_top_level/js/encode', 'sharedhelpers/storageProvider', 'models/loyaltyAccountModel', 'helpers/theatersHelper'], function () {
                    var callback = function () {
                        var storage = new StorageProvider();
                        storage.read('LoyaltyCard', function (value) {
                            if (value) {
                                var loyaltyStoreModel = {
                                    number: self.model.get('id'),
                                    firstName: $.trim(self.model.get('firstName').value),
                                    lastName: $.trim(self.model.get('lastName').value),
                                    offers: self.model.get('loyaltyBanners') || null,
                                    multitype: value.multitype || false,
                                    favoriteTheater: self.model.get('favoriteTheater').value,
                                    password: self.model.get('password').value,
                                    smsOptIn: self.model.get('smsOptIn').value
                                };

                                if (window.appConfig.EnableGenderForLoyalty) {
                                    loyaltyStoreModel.gender = self.model.get('gender').value;
                                }

                                storage.store('LoyaltyCard', loyaltyStoreModel);
                            }
                        });

                        var theater = _.find(window.ObjectCollections.TheaterCollection.models, function (model) {
                            return model.get('name') == self.model.get('favoriteTheater').value;
                        });

                        if (theater) {
                            var theatersHelper = new TheatersHelper();
                            theatersHelper.markTheaterAsFavorite(theater.id, true);
                        }
                        window.AppMethods.ChangePage("#loyaltycard");
                    };

                    window.showPageLoadingMessage();
                    $('[data-role=content]').css({ 'pointer-events': 'none' });
                    $('#saveLoyaltyCard').css({ 'pointer-events': 'none' });

                    self.model.saveCustom({ isUpdate: true, callback: callback });

                });
            }
        },

        deleteLoyaltyCard: function (event) {
            event.preventDefault();
            var self = this;

            var deleteCard = function () {
                var onConfirm = function (button) {
                    if (button == 1) {
                        var stor = new StorageProvider();
                        stor.remove("LoyaltyCard");
                        window.loyaltyCard = null;
                        window.AppMethods.ChangePage('');
                    }
                };

                if (window.isMobileDevice()) {
                    navigator.notification.confirm(self.localize.removeLoyaltyCardFromApplication, onConfirm, self.localize.alertTitle, self.localize.ok + ', ' + self.localize.cancel);
                } else if (confirm(self.localize.removeLoyaltyCardFromApplication)) {
                    onConfirm(1);
                }
            };

            if (!configurationProvider.useDualCards()) {
                deleteCard();
                return;
            }

            require(['helpers/paymentMethodHelper'], function () {
                var paymentMethod = new PaymentMethodsHelper();

                paymentMethod.get('', function (paymentMethods) {
                    var giftCard = _.find(paymentMethods, function (card) { return card.CardNumber == self.model.get('id'); });

                    if (giftCard) {
                        require(['helpers/dualCardHelper'], function () {
                            var callback = function () {
                                window.loyaltyCard = null;
                                window.AppMethods.ChangePage('');
                            };

                            window.dualCardHelper.deleteCard(giftCard.Id, callback);
                        });

                        return;
                    }

                    deleteCard();
                });
            });
        },

        zipcode: function () {
            if (ConfigHelper.isDeviceOnline() == false) {
                window.showAlert(this.localize.errorUnableConnectToInternet);
                return;
            }

            if (!this.zipValidation.searchZip)
                return;

            var zipCode = $('#zipcode');
            zipCode = zipCode.val() !== undefined ? zipCode.val().trim() : zipCode.val();

            var args = {
                zipCode: zipCode,
                successCallback: function (data) {
                    $("#stateProvince option[value=" + data.result[0].State + "]").attr('selected', 'selected');
                    $('#city').val(data.result[0].City);
                    $('#stateProvince').selectmenu("refresh");
                }
            };

            $('#city, #stateProvince, #zipcode').addClass('field-updated');

            this.zipValidation.searchZip(args);
        },


        handlePasswordValidation: function () {
            var el = $(this.el);

            var currentPass = el.find('#currentPass').val();
            var newPass = el.find('#pass').val();
            var repeatPass = el.find('#repeatPass').val();

            el.find('.error[for="currentPass"]').remove();
            el.find('.error[for="pass"]').remove();
            el.find('.error[for="repeatPass"]').remove();

            el.find("#currentPass, #pass, #repeatPass").removeClass('error');

            if (currentPass == '' && (newPass != '' || repeatPass != '')) {
                el.find("#currentPass").addClass('error');
                $('<label class="error" for="currentPass" generated="true">' + this.localize.errorLoyaltyMissingCurrentPassword + '</label>').insertAfter(el.find("#currentPass").parent());
                return false;

            } else if (currentPass != '' && newPass == '') {
                el.find("#pass").addClass('error');
                $('<label class="error" for="pass" generated="true">' + this.localize.required + '</label>').insertAfter(el.find("#pass").parent());
                return false;

            } else if (currentPass != '' && repeatPass == '') {
                el.find("#repeatPass").addClass('error');
                $('<label class="error" for="repeatPass" generated="true">' + this.localize.required + '</label>').insertAfter(el.find("#repeatPass").parent());
                return false;

            } else if (currentPass && newPass && repeatPass && currentPass == newPass) {
                el.find("#pass, #repeatPass").addClass('error');
                $('<label class="error" for="pass" generated="true">' + this.localize.errorLoyaltyChooseNewPassword + '</label>').insertAfter(el.find("#pass").parent());
                $('<label class="error" for="repeatPass" generated="true">' + this.localize.errorLoyaltyChooseNewPassword + '</label>').insertAfter(el.find("#repeatPass").parent());

                return false;

            }
            return true;
        },

        changePassword: function () {
            var that = this;
            var frmChangePassword = $('#frmChangePassword');
            frmChangePassword.validate().resetForm();
            if (frmChangePassword.valid() && that.handlePasswordValidation()) {
                window.showPageLoadingMessage();

                require(['shared_top_level/js/encode', 'sharedhelpers/storageProvider', 'models/loyaltyAccountModel'], function () {
                    var storage = new StorageProvider();
                    var encryptionHelper = new EncryptionHelper();

                    storage.read('LoyaltyCard', function (value) {
                        if (value) {
                            var cardModel = new LoyaltyAccountModel();
                            if (window.appConfig.LoyaltyPasswordEncryptionDisabled) {
                                cardModel.set({ id: value.number, securityKey: encodeURIComponent($('#currentPass').val()) });
                            } else {
                                cardModel.set({ id: value.number, securityKey: encodeURIComponent(encryptionHelper.encrypt($('#currentPass').val())) });
                            }
                            cardModel.fetch({
                                cache: false,
                                success: function () {
                                    window.hidePageLoadingMessage();
                                    var modelCardNumber = cardModel.get('cardNumber') !== undefined ? cardModel.get('cardNumber').value : null;

                                    if (modelCardNumber === null) {
                                        window.showAlert(that.localize.errorLoyaltyPasswordNotExists);
                                        return;
                                    } else {
                                        if (window.appConfig.LoyaltyPasswordEncryptionDisabled) {
                                            cardModel.get('password').value = $('#repeatPass').val();
                                        } else {
                                            cardModel.get('password').value = encryptionHelper.encrypt($('#repeatPass').val());
                                        }
                                        
                                        cardModel.get('password').isUpdated = true;

                                        var callback = function () {
                                            storage.read('LoyaltyCard', function (loyaltyStoreModel) {
                                                if (loyaltyStoreModel) {

                                                    loyaltyStoreModel.password = cardModel.get('password').value;
                                                    storage.store('LoyaltyCard', loyaltyStoreModel);
                                                }
                                            });
                                            window.AppMethods.ChangePage("#loyaltycard");
                                        };

                                        window.showPageLoadingMessage();
                                        cardModel.saveCustom({ isUpdate: true, callback: callback });
                                    }
                                },
                                error: function () {
                                    window.hidePageLoadingMessage();
                                    window.showAlert(that.localize.errorGettingLoyaltyCardData);
                                }
                            });
                        }

                    });
                });
            }
        },

        switchTabs: function (e) {
            e.preventDefault();
            if (isJqmGhostClick(e)) return;

            var currentTab = e.currentTarget.attributes['value'].value;

            $(this.el).find('#' + currentTab).show();
            $(this.el).find('.loyalty-tab').not('#' + currentTab).hide();
        }
    });

    return view;
});