define([
  'sharedhelpers/configurationProvider',
  'text!templates/SettingsView.html',
  'json!js/versionInfo.json',
  'sharedhelpers/storageProvider',
  'sharedhelpers/cultureHelper',
  'facebook'
], function(configurationProvider, template, versionInfo) {
  var view = Backbone.View.extend({
    template: _.template(template),
    localize: window.ObjectCollections.Localization.result,

    initialize: function() {
      this.settingsEmailKey = 'emailAddress';
      this.settingsPhoneKey = 'phoneNumber';
      this.settingsFbKey = 'fbAccount';
      this.settingsHistoryCleanup = 'HistoryCleanup';
      this.settingsTouchID = 'TouchID';
      this.facebook = new FacebookHelper();
      this.storage = new StorageProvider();
    },

    render: function() {
      var that = this;
      that.$el.html(that.template({
        localize: that.localize,
        configurationProvider: configurationProvider,
        hideBackButton: that.options.hideBackButton,
        isTouchIdAvailable: window.AppProperties.isTouchIdAvailable,
        versionNumber: versionInfo.versionNumber
      }));

      that.loadSettings(that.$el);
      var retView = configurationProvider.arangeFlow(that);

      document.addEventListener('showkeyboard', that.showKeyboardHandler, false);
      document.addEventListener('hidekeyboard', that.hideKeyboardHandler, false);

      var cultureHelper = new CultureHelper();
      var phoneValidation = cultureHelper.phoneValidation();
      var emailValidation = cultureHelper.emailValidation();

      $.validator.addMethod('phone', function(value) {
        return value == "" || phoneValidation.regex.test(value);
      }, phoneValidation.msg);

      $.validator.addMethod('email', function(value) {
        return value == "" || emailValidation.regex.test(value);
      }, emailValidation.msg);

      return retView;
    },

    events: {
      'blur #email': 'saveSettings',
      'blur #phone': 'saveSettings',
      'tap .connect-to-facebook-button': 'onLogin',
      'tap #facebook-account-disconnect': 'onDisconnect',
      "pagebeforehide ": "onPageHide",
      'tap #paymentMethodSettings': 'navigatePaymentSettings',
      'tap #locationSettings': 'navigateLocationSettings',
      'tap #privacy-policy': 'privacyPolicy'
    },

    onPageHide: function() {
      var that = this;
      document.removeEventListener('hidekeyboard', that.hideKeyboardHandler);
      document.removeEventListener('showkeyboard', that.showKeyboardHandler);

      var historyCleanup = $('#historyCleanup').val() == "yes";

      if (historyCleanup) {
        that.storage.store(that.settingsHistoryCleanup, true);
      } else {
        that.storage.read(that.settingsHistoryCleanup, function(value) {
          if (value === true || value === false) {
            that.storage.store(that.settingsHistoryCleanup, false);
          }
        });
      }

      var touchID = $('#touchID').val() == "yes";

      if (touchID) {
        that.storage.store(that.settingsTouchID, true);
      } else {
        that.storage.read(that.settingsTouchID, function(value) {
          if (value === true || value === false) {
            that.storage.store(that.settingsTouchID, false);
          }
        });
      }
    },

    showKeyboardHandler: function() {
      $('.app-version-footer').hide();
    },

    hideKeyboardHandler: function() {
      $('input').blur();
      $('.app-version-footer').show();
    },

    navigatePaymentSettings: function(e) {
      e.preventDefault();
      if (isJqmGhostClick(e) || isChangePageFired(e)) return;

      window.showPageLoadingMessage();

      _.defer(function() {
        window.AppMethods.ChangePage('#paymentMethodSettings', false, true);
      });
    },

    navigateLocationSettings: function(e) {
      e.preventDefault();
      if (isJqmGhostClick(e) || isChangePageFired(e)) return;

      window.showPageLoadingMessage();

      _.defer(function() {
        window.AppMethods.ChangePage('#locationSettings', false, true);
      });
    },

    onLogin: function(e) {
      e.preventDefault();
      if (isJqmGhostClick(e)) return;
      var that = this;

      if (ConfigHelper.isDeviceOnline() == false) {
        window.showAlert(that.localize.errorUnableConnectToInternet);
        return;
      }

      if (window.facebookConnectPlugin) {
        window.facebookConnectPlugin.getLoginStatus(function(response) {
          if (response.status === "connected") {
            that.getFBProfile(response.authResponse.accessToken);
          } else {
            window.facebookConnectPlugin.login(["public_profile"], function(result) {
              that.getFBProfile(result.authResponse.accessToken);
            });
          }
        });
      }
    },

    getFBProfile: function(accessToken) {
      var that = this;
      if (ConfigHelper.isDeviceOnline() == false) {
        window.showAlert(that.localize.errorUnableConnectToInternet);
        return;
      }
      var callback = function(result) {
        $(that.el).find('.fb-friend-img').attr('src', 'https://graph.facebook.com/' + result.id + '/picture');
        $(that.el).find('#fb-account-name').html(result.name);
        $(that.el).find('.profile-container').show();
        $(that.el).find('.connect-to-facebook-button').hide();

        var fb = {
          accessToken: accessToken,
          id: result.id
        };

        that.storage.store(that.settingsFbKey, fb);
      };
      window.showPageLoadingMessage();

      that.facebook.getData(accessToken, {
        action: "me?"
      }, function(result) {
        if (result) {
          callback(result);
        }
        window.hidePageLoadingMessage();
      });
    },

    onDisconnect: function(e) {
      e.preventDefault();
      if (isJqmGhostClick(e) || isChangePageFired(e)) return;

      var self = this;

      window.facebookConnectPlugin.logout(function() {
        $('.fb-friend-img').attr('src', '');
        $('#fb-account-name').empty();
        $('.profile-container').hide();
        $('.connect-to-facebook-button').show();
        window.AppProperties.tempFbAccessToken = null;
        self.storage.remove(self.settingsFbKey);
      });
    },

    loadSettings: function(selector) {
      var self = this;

      window.showPageLoadingMessage();
      if (window.facebookConnectPlugin) {
        window.facebookConnectPlugin.getLoginStatus(function(response) {
          if (response.status === "connected") {
            self.getFBProfile(response.authResponse.accessToken);
          } else {
            selector.find('.connect-to-facebook-button').show();
          }
        });
      }

      self.storage.read(self.settingsEmailKey, function(value) {
        if (value) {
          selector.find('#email').val(value);
          self.email = value;
        }
      });

      self.storage.read(self.settingsPhoneKey, function(value) {
        if (value) {
          selector.find('#phone').val(value);
          self.phone = value;
        }
      });

      self.storage.read(self.settingsHistoryCleanup, function(value) {
        var option = value ? 'yes' : 'no';
        selector.find('#historyCleanup').val(option).slider('refresh');
      });

      self.storage.read(self.settingsTouchID, function(value) {
        var option = value ? 'yes' : 'no';
        selector.find('#touchID').val(option).slider('refresh');
      });

      window.hidePageLoadingMessage();
    },

    saveSettings: function() {
      var email = $(this.el).find('#email');
      var phone = $(this.el).find('#phone');

      if (phone.val() !== '') {
        if ($(this.el).find('#frmPhone').valid()) {
          this.storage.store(this.settingsPhoneKey, phone.val());
          this.phone = phone.val();
        }
      } else {
        if (phone.val() !== this.phone && this.phone !== undefined) {
          this.storage.remove(this.settingsPhoneKey);
          this.phone = undefined;
        }
      }

      if (email.val() !== '') {
        if ($(this.el).find('#frmEmail').valid()) {
          this.storage.store(this.settingsEmailKey, email.val());
          this.email = email.val();
        }
      } else {
        if (email.val() !== this.email && this.email !== undefined) {
          this.storage.remove(this.settingsEmailKey);
          this.email = undefined;
        }
      }
    },

    notification: function(text) {
      window.showAlert(text);
    },

    privacyPolicy: function(e) {
      e.preventDefault();
      if (isJqmGhostClick(e)) return;

      var privacyUrl = configurationProvider.privacyPolicy();

      var inAppWin = window.open(privacyUrl, '_blank');

      inAppWin.addEventListener('exit', function() {
        inAppWin.addEventListener('loadstop');
        inAppWin.addEventListener('exit');
        inAppWin = null;
      });
    }
  });

  return view;
});
